@@ -28,18 +28,23 @@ __author__ = "Valentin Valls"
 __contact__ = "valentin.valls@esrf.eu"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "14/05/2019"
+__date__ = "21/01/2020"
 __status__ = "production"
 
-import logging
 import os
 import sys
 import datetime
 from argparse import ArgumentParser
 
+import logging
 logging.basicConfig(level=logging.INFO)
 logging.captureWarnings(True)
-logger = logging.getLogger("pyFAI-calib2")
+logger = logging.getLogger(__name__)
+try:
+    import hdf5plugin  # noqa
+except ImportError:
+    logger.debug("Unable to load hdf5plugin, backtrace:", exc_info=True)
+
 logger_uncaught = logging.getLogger("pyFAI-calib2.UNCAUGHT")
 
 import pyFAI.resources
@@ -48,7 +53,6 @@ import pyFAI.detectors
 import pyFAI.io.image
 from pyFAI.io.ponifile import PoniFile
 
-
 try:
     from rfoo.utils import rconsole
     rconsole.spawn_server()
