@@ -32,13 +32,11 @@ ControlPoints: a set of control points associated with a calibration image
 PointGroup: a group of points
 """
 
-from __future__ import absolute_import, print_function, with_statement, division
-
 __author__ = "Jérôme Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "06/03/2018"
+__date__ = "08/01/2021"
 __status__ = "development"
 __docformat__ = 'restructuredtext'
 
@@ -51,7 +49,6 @@ import logging
 import numpy
 import array
 
-from pyFAI.third_party import six
 from .calibrant import Calibrant, get_calibrant, names as calibrant_names
 
 logger = logging.getLogger(__name__)
@@ -62,6 +59,7 @@ class ControlPoints(object):
     This class contains a set of control points with (optionally) their
     ring number hence d-spacing and diffraction  2Theta angle...
     """
+
     def __init__(self, filename=None, calibrant=None, wavelength=None):
         self._sem = threading.Semaphore()
         self._groups = OrderedDict()
@@ -74,7 +72,7 @@ class ControlPoints(object):
         if (not have_spacing) and (calibrant is not None):
             if isinstance(calibrant, Calibrant):
                 self.calibrant = calibrant
-            elif type(calibrant) in six.string_types:
+            elif type(calibrant) in (str,):
                 if calibrant in calibrant_names():
                     self.calibrant = get_calibrant(calibrant)
                 elif os.path.isfile(calibrant):
@@ -242,69 +240,70 @@ class ControlPoints(object):
         wavelength = None
         dspacing = []
 
-        for line in open(filename, "r"):
-            if line.startswith("#"):
-                continue
-            elif ":" in line:
-                key, value = line.split(":", 1)
-                value = value.strip()
-                key = key.strip().lower()
-                if key == "calibrant":
-                    words = value.split()
-                    if words[0] in calibrant_names():
-                        calibrant = get_calibrant(words[0])
-                    try:
-                        wavelength = float(words[-1])
-                        calibrant.set_wavelength(wavelength)
-                    except Exception as error:
-                        logger.error("ControlPoints.load: unable to convert to float %s (wavelength): %s", value, error)
-                elif key == "wavelength":
-                    try:
-                        wavelength = float(value)
-                    except Exception as error:
-                        logger.error("ControlPoints.load: unable to convert to float %s (wavelength): %s", value, error)
-                elif key == "dspacing":
-                    for val in value.split():
+        with open(filename, 'r') as opened_file:
+            for line in opened_file:
+                if line.startswith("#"):
+                    continue
+                elif ":" in line:
+                    key, value = line.split(":", 1)
+                    value = value.strip()
+                    key = key.strip().lower()
+                    if key == "calibrant":
+                        words = value.split()
+                        if words[0] in calibrant_names():
+                            calibrant = get_calibrant(words[0])
                         try:
-                            fval = float(val)
-                        except Exception:
-                            fval = None
-                        dspacing.append(fval)
-                elif key == "ring":
-                    if value.lower() == "none":
-                        ring = None
-                    else:
-                        try:
-                            ring = int(value)
+                            wavelength = float(words[-1])
+                            calibrant.set_wavelength(wavelength)
                         except Exception as error:
-                            logger.error("ControlPoints.load: unable to convert to int %s (ring): %s", value, error)
-                elif key == "point":
-                    vx = None
-                    vy = None
-                    if "x=" in value:
-                        vx = value[value.index("x=") + 2:].split()[0]
-                    if "y=" in value:
-                        vy = value[value.index("y=") + 2:].split()[0]
-                    if (vx is not None) and (vy is not None):
+                            logger.error("ControlPoints.load: unable to convert to float %s (wavelength): %s", value, error)
+                    elif key == "wavelength":
                         try:
-                            x = float(vx)
-                            y = float(vy)
+                            wavelength = float(value)
                         except Exception as error:
-                            logger.error("ControlPoints.load: unable to convert to float %s (point): %s", value, error)
-                        else:
-                            points.append([y, x])
-                elif key.startswith("new"):
-                    if len(points) > 0:
-                        with self._sem:
-                            gpt = PointGroup(points, ring)
-                            self._groups[gpt.label] = gpt
-                            points = []
+                            logger.error("ControlPoints.load: unable to convert to float %s (wavelength): %s", value, error)
+                    elif key == "dspacing":
+                        for val in value.split():
+                            try:
+                                fval = float(val)
+                            except Exception:
+                                fval = None
+                            dspacing.append(fval)
+                    elif key == "ring":
+                        if value.lower() == "none":
                             ring = None
-                elif key in ["2theta"]:
-                    # Deprecated keys
-                    pass
-                else:
-                    logger.error("Unknown key: %s", key)
+                        else:
+                            try:
+                                ring = int(value)
+                            except Exception as error:
+                                logger.error("ControlPoints.load: unable to convert to int %s (ring): %s", value, error)
+                    elif key == "point":
+                        vx = None
+                        vy = None
+                        if "x=" in value:
+                            vx = value[value.index("x=") + 2:].split()[0]
+                        if "y=" in value:
+                            vy = value[value.index("y=") + 2:].split()[0]
+                        if (vx is not None) and (vy is not None):
+                            try:
+                                x = float(vx)
+                                y = float(vy)
+                            except Exception as error:
+                                logger.error("ControlPoints.load: unable to convert to float %s (point): %s", value, error)
+                            else:
+                                points.append([y, x])
+                    elif key.startswith("new"):
+                        if len(points) > 0:
+                            with self._sem:
+                                gpt = PointGroup(points, ring)
+                                self._groups[gpt.label] = gpt
+                                points = []
+                                ring = None
+                    elif key in ["2theta"]:
+                        # Deprecated keys
+                        pass
+                    else:
+                        logger.error("Unknown key: %s", key)
         if len(points) > 0:
             with self._sem:
                 gpt = PointGroup(points, ring)
@@ -358,33 +357,33 @@ class ControlPoints(object):
         """
         Ask the ring number values for the given points
         """
-        lastRing = None
+        last_ring = None
         lst = list(self._groups.keys())
         lst.sort(key=lambda item: self._groups[item].code)
         for lbl in lst:
-            bOk = False
+            is_ok = False
             gpt = self._groups[lbl]
-            while not bOk:
-                defaultRing = 0
+            while not is_ok:
+                default_ring = 0
                 ring = gpt.ring
                 if ring is not None:
-                    defaultRing = ring
-                elif lastRing is not None:
-                    defaultRing = lastRing + 1
+                    default_ring = ring
+                elif last_ring is not None:
+                    default_ring = last_ring + 1
                 msg = "Point group #%2s (%i points)\t (%6.1f,%6.1f) \t [default=%s] Ring# "
-                res = six.moves.input(msg % (lbl, len(gpt), gpt.points[0][1], gpt.points[0][0], defaultRing))
+                res = input(msg % (lbl, len(gpt), gpt.points[0][1], gpt.points[0][0], default_ring))
                 res = res.strip()
                 if res == "":
-                    res = defaultRing
+                    res = default_ring
                 try:
                     input_ring = int(res)
                 except (ValueError, TypeError):
                     logging.error("I did not understand the ring number you entered")
                 else:
                     if input_ring >= 0 and input_ring < len(self.calibrant.dSpacing):
-                        lastRing = ring
+                        last_ring = ring
                         gpt.ring = input_ring
-                        bOk = True
+                        is_ok = True
                     else:
                         logging.error("Invalid ring number %i (range 0 -> %2i)",
                                       input_ring, len(self.calibrant.dSpacing) - 1)
