@@ -1,5 +1,5 @@
 #
-#    Copyright (C) 2017-2018 European Synchrotron Radiation Facility, Grenoble, France
+#    Copyright (C) 2017-2021 European Synchrotron Radiation Facility, Grenoble, France
 #
 #  Permission is hereby granted, free of charge, to any person obtaining a copy
 #  of this software and associated documentation files (the "Software"), to deal
@@ -19,24 +19,23 @@
 #  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 #  THE SOFTWARE.
 
-
 """CSR rebinning engine implemented in pure python (with bits of scipy !) 
 """
 
-from __future__ import absolute_import, print_function, with_statement
-
 __author__ = "Jerome Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "02/01/2020"
+__date__ = "28/01/2022"
 __status__ = "development"
 
 import logging
+import warnings
 logger = logging.getLogger(__name__)
 import numpy
 from scipy.sparse import csr_matrix
 from .preproc import preproc as preproc_np
+from ..utils.mathutil import interp_filter
 try:
     from ..ext.preproc import preproc as preproc_cy
 except ImportError as err:
@@ -44,11 +43,12 @@ except ImportError as err:
     preproc = preproc_np
 else:
     preproc = preproc_cy
-
+from ..utils import calc_checksum
 from ..containers import Integrate1dtpl, Integrate2dtpl
 
 
 class CSRIntegrator(object):
+
     def __init__(self,
                  image_size,
                  lut=None,
@@ -64,7 +64,7 @@ class CSRIntegrator(object):
         self.empty = empty
         self.bins = None
         self._csr = None
-        self._csr2 = None # Used for propagating variance
+        self._csr2 = None  # Used for propagating variance
         self.lut_size = 0  # actually nnz
         self.data = None
         self.indices = None
@@ -84,13 +84,13 @@ class CSRIntegrator(object):
         self.indptr = indptr
         self.lut_size = len(indices)
         self.bins = len(indptr) - 1
-        print(self.bins, self.size)
         self._csr = csr_matrix((data, indices, indptr), shape=(self.bins, self.size))
-        self._csr2 = csr_matrix((data * data, indices, indptr), shape=(self.bins, self.size)) # contains the coef squared, used for variance propagation
+        self._csr2 = csr_matrix((data * data, indices, indptr), shape=(self.bins, self.size))  # contains the coef squared, used for variance propagation
 
     def integrate(self,
                   signal,
                   variance=None,
+                  poissonian=None,
                   dummy=None,
                   delta_dummy=None,
                   dark=None,
@@ -104,6 +104,7 @@ class CSRIntegrator(object):
         
         :param signal: array of the right size with the signal in it.
         :param variance: Variance associated with the signal
+        :param poissonian: set to use signal as variance (minimum 1), set to False to use azimuthal model. 
         :param dummy: values which have to be discarded (dynamic mask)
         :param delta_dummy: precision for dummy values
         :param dark: noise to be subtracted from signal
@@ -131,26 +132,35 @@ class CSRIntegrator(object):
                        empty=self.empty,
                        split_result=4,
                        variance=variance,
-                       dtype=numpy.float32)
+                       dtype=numpy.float32,
+                       poissonian=bool(poissonian))
         prep.shape = numpy.prod(shape), -1
-        logger.warning("prep.shape %s lut_size %s, image_size %s, bins %s", prep.shape, self.lut_size, self.size, self.bins)
+        # logger.warning("prep.shape %s lut_size %s, image_size %s, bins %s", prep.shape, self.lut_size, self.size, self.bins)
         res = numpy.empty((numpy.prod(self.bins), 4), dtype=numpy.float32)
-        logger.warning(self._csr.shape)
+        # logger.warning(self._csr.shape)
         res[:, 0] = self._csr.dot(prep[:, 0])
-        if variance is not None:
-            res[:, 1] = self._csr2.dot(prep[:, 1])
         res[:, 2] = self._csr.dot(prep[:, 2])
         res[:, 3] = self._csr.dot(prep[:, 3])
+        if variance is not None or poissonian:
+            res[:, 1] = self._csr2.dot(prep[:, 1])
+        elif poissonian is False:
+            # ask for azimuthal error model
+            avg = res[:, 0] / res[:, 2]
+            avg_ext = self._csr.T.dot(avg)
+            delta = (prep[:, 0] / prep[:, 2] - avg_ext) ** 2
+            res[:, 1] = self._csr.dot(delta)
         return res
 
 
 class CsrIntegrator1d(CSRIntegrator):
+
     def __init__(self,
                  image_size,
                  lut=None,
                  empty=0.0,
                  unit=None,
                  bin_centers=None,
+                 mask_checksum=None
                  ):
         """Constructor of the abstract class for 1D integration
         
@@ -159,31 +169,14 @@ class CsrIntegrator1d(CSRIntegrator):
         :param empty: value for empty pixels
         :param unit: the kind of radial units
         :param bin_center: position of the bin center
-        
-        Nota: bins are deduced from bin_centers 
-
-
-        TODO: 
-        ~/workspace-400/pyFAI/build/lib.linux-x86_64-3.7/pyFAI/azimuthalIntegrator.py in sigma_clip_ng(self, data, npt, correctSolidAngle, polarization_factor, variance, error_model, dark, flat, method, unit, thres, max_iter, dummy, delta_dummy, mask, normalization_factor, metadata, safe, **kwargs)
-   3508                         elif (mask is None) and (integr.check_mask):
-   3509                             reset = "no mask but CSR has mask"
--> 3510                         elif (mask is not None) and (integr.mask_checksum != mask_crc):
-   3511                             reset = "mask changed"
-   3512 #                         if (radial_range is None) and (integr.pos0Range is not None):
-
-AttributeError: 'CsrIntegrator1d' object has no attribute 'mask_checksum'
-
+        :param mask_checksum: just a place-holder to track which mask was used
+        Nota: bins value is deduced from the dimentionality of bin_centers 
         """
         self.bin_centers = bin_centers
         CSRIntegrator.__init__(self, image_size, lut, empty)
-        self.pos0_range = self.pos1_range = self._geometry = None
+        self.pos0_range = self.pos1_range = None
         self.unit = unit
-
-    def set_geometry(self, geometry):
-        from pyFAI.geometry import Geometry
-        assert numpy.prod(geometry.detector.shape) == self.size
-        assert isinstance(geometry, Geometry)
-        self._geometry = geometry
+        self.mask_checksum = mask_checksum
 
     def set_matrix(self, data, indices, indptr):
         """Actually set the CSR sparse matrix content
@@ -198,6 +191,7 @@ AttributeError: 'CsrIntegrator1d' object has no attribute 'mask_checksum'
     def integrate(self,
                   signal,
                   variance=None,
+                  poissonian=None,
                   dummy=None,
                   delta_dummy=None,
                   dark=None,
@@ -211,6 +205,7 @@ AttributeError: 'CsrIntegrator1d' object has no attribute 'mask_checksum'
         
         :param signal: array of the right size with the signal in it.
         :param variance: Variance associated with the signal
+        :param poissonian: set to use signal as variance (minimum 1), set to False to use azimuthal model.
         :param dummy: values which have to be discarded (dynamic mask)
         :param delta_dummy: precision for dummy values
         :param dark: noise to be subtracted from signal
@@ -222,11 +217,12 @@ AttributeError: 'CsrIntegrator1d' object has no attribute 'mask_checksum'
         :return: Integrate1dResult or Integrate1dWithErrorResult object depending on variance 
         
         """
-        if variance is None:
+        if variance is None and poissonian is None:
             do_variance = False
         else:
             do_variance = True
-        trans = CSRIntegrator.integrate(self, signal, variance, dummy, delta_dummy,
+        trans = CSRIntegrator.integrate(self, signal, variance, poissonian,
+                                        dummy, delta_dummy,
                                         dark, flat, solidangle, polarization,
                                         absorption, normalization_factor)
         signal = trans[:, 0]
@@ -245,8 +241,9 @@ AttributeError: 'CsrIntegrator1d' object has no attribute 'mask_checksum'
         return Integrate1dtpl(self.bin_centers,
                               intensity, error,
                               signal, variance, normalization, count)
+
     integrate_ng = integrate
-    
+
     def sigma_clip(self, data, dark=None, dummy=None, delta_dummy=None,
                    variance=None, dark_variance=None,
                    flat=None, solidangle=None, polarization=None, absorption=None,
@@ -272,6 +269,9 @@ AttributeError: 'CsrIntegrator1d' object has no attribute 'mask_checksum'
         Integration is performed using the CSR representation of the look-up table on all
         arrays: signal, variance, normalization and count
 
+        Formula for azimuthal variance from:
+        https://dbs.ifi.uni-heidelberg.de/files/Team/eschubert/publications/SSDBM18-covariance-authorcopy.pdf
+
         :param dark: array of same shape as data for pre-processing
         :param dummy: value for invalid data
         :param delta_dummy: precesion for dummy assessement
@@ -280,7 +280,7 @@ AttributeError: 'CsrIntegrator1d' object has no attribute 'mask_checksum'
         :param flat: array of same shape as data for pre-processing
         :param solidangle: array of same shape as data for pre-processing
         :param polarization: array of same shape as data for pre-processing
-        :param safe: if True (default) compares arrays on GPU according to their checksum, unless, use the buffer location is used
+        :param safe: Unused in this implementation
         :param normalization_factor: divide raw signal by this value
         :param cutoff: discard all points with |value - avg| > cutoff * sigma. 3-4 is quite common 
         :param cycle: perform at maximum this number of cycles. 5 is common.
@@ -288,10 +288,10 @@ AttributeError: 'CsrIntegrator1d' object has no attribute 'mask_checksum'
         """
         shape = data.shape
         error_model = error_model.lower() if error_model else ""
-        
-        if self._geometry is None:
-            raise RuntimeError("Set geometry first")
-        
+
+        poissonian = error_model.startswith("pois")
+        azimuthal = (not poissonian) and (variance is None)
+
         prep = preproc(data,
                        dark=dark,
                        flat=flat,
@@ -305,43 +305,97 @@ AttributeError: 'CsrIntegrator1d' object has no attribute 'mask_checksum'
                        empty=self.empty,
                        split_result=4,
                        variance=variance,
+                       dark_variance=dark_variance,
                        dtype=numpy.float32,
-                       poissonian=error_model.startswith("pois"))
+                       poissonian=poissonian)
+
         prep_flat = prep.reshape((numpy.prod(shape), 4))
-        res = self._csr.dot(prep_flat)
-        print(cycle)
+        res = numpy.empty((numpy.prod(self.bins), 4), dtype=numpy.float32)
+
+        # First azimuthal integration:
+        res[:, 0] = self._csr.dot(prep_flat[:, 0])
+        res[:, 2] = self._csr.dot(prep_flat[:, 2])
+        res[:, 3] = self._csr.dot(prep_flat[:, 3])
+        if azimuthal:
+            # ask for azimuthal error model
+            with warnings.catch_warnings():
+                warnings.simplefilter("ignore")
+                avg = res[:, 0] / res[:, 2]
+                avg_ext = self._csr.T.dot(interp_filter(avg, avg))  # backproject the average value to the image
+                norm = prep_flat[:, 2]
+                msk = (norm == 0)
+                delta2 = (prep_flat[:, 0] / norm - avg_ext) ** 2
+                delta2[msk] = 0
+            res[:, 1] = self._csr.dot(delta2)  # * norm) / self._csr.dot(norm)
+        else:
+            res[:, 1] = self._csr2.dot(prep_flat[:, 1])
+
         for _ in range(cycle):
-            msk = res[:, 2] == 0
-            avg = res[:, 0] / res[:, 2]
-            std = numpy.sqrt(res[:, 1] / res[:, 2])
-            avg[msk] = 0
-            std[msk] = 0
-            
-            avg2d = self._geometry.calcfrom1d(self.bin_centers, avg, shape=shape,
-                    dim1_unit=self.unit, correctSolidAngle=False, dummy=0.0)
-            std2d = self._geometry.calcfrom1d(self.bin_centers, std, shape=shape,
-                    dim1_unit=self.unit, correctSolidAngle=False, dummy=0.0)
-            cnt = abs(prep[..., 0]/prep[..., 2] - avg2d)/std2d
-            msk2d = numpy.logical_and(numpy.logical_not(numpy.isfinite(cnt)), cnt> cutoff)
-            prep[msk2d, :] = 0
-            res = self._csr.dot(prep_flat)
-        msk = res[:, 2] == 0
-        avg = res[:, 0] / res[:, 2]
-        std = numpy.sqrt(res[:, 1] / res[:, 2])
-        avg[msk] = 0
-        std[msk] = 0        
+            nrm = res[:, 2]
+            with warnings.catch_warnings():
+                warnings.simplefilter("ignore")
+                avg = res[:, 0] / nrm
+                std = numpy.sqrt(res[:, 1] / nrm)
+            # Replace NaNs with interpolated values.
+            avg = interp_filter(avg, avg)
+            std = interp_filter(std, std)
+            cnt = numpy.maximum(res[:, 3], 3)
+
+            # Interpolate in 2D:
+            avg2d = self._csr.T.dot(avg)
+            std2d = self._csr.T.dot(std)
+            cnt2d = numpy.maximum(self._csr.T.dot(cnt), 3)  # Needed for Chauvenet criterion
+            with warnings.catch_warnings():
+                warnings.simplefilter("ignore")
+                delta = abs(prep_flat[..., 0] / prep_flat[..., 2] - avg2d)
+            chauvenet = numpy.maximum(cutoff, numpy.sqrt(2.0 * numpy.log(cnt2d / numpy.sqrt(2.0 * numpy.pi)))) * std2d
+            # chauvenet = cutoff * std2d
+            msk2d = numpy.where(numpy.logical_not(delta <= chauvenet))
+            # print(len(msk2d[0]))
+            prep_flat[msk2d] = 0
+            # subsequent integrations:
+            res[:, 0] = self._csr.dot(prep_flat[:, 0])
+            res[:, 2] = self._csr.dot(prep_flat[:, 2])
+            res[:, 3] = self._csr.dot(prep_flat[:, 3])
+            if azimuthal:
+                with warnings.catch_warnings():
+                    warnings.simplefilter("ignore")
+                    avg = res[:, 0] / res[:, 2]
+                    avg_ext = self._csr.T.dot(interp_filter(avg, avg))  # backproject the average value to the image
+                    msk = prep_flat[:, 2] == 0
+                    delta2 = (prep_flat[:, 0] / prep_flat[:, 2] - avg_ext) ** 2
+                    delta2[msk] = 0
+                res[:, 1] = self._csr.dot(delta2)
+            else:
+                res[:, 1] = self._csr2.dot(prep_flat[:, 1])
+
+        nrm = res[:, 2]
+        msk = nrm <= 0
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore")
+            avg = res[:, 0] / nrm
+            # Here we return the standaard deviation and not the standard error of the mean !
+            std = numpy.sqrt(res[:, 1]) / nrm
+        avg[msk] = self.empty
+        std[msk] = self.empty
+
+        return Integrate1dtpl(self.bin_centers, avg, std, res[:, 0], res[:, 1], res[:, 2], res[:, 3])
+
+    @property
+    def check_mask(self):
+        return self.mask_checksum is not None
 
-        return Integrate1dtpl(self.bin_centers,avg, std, res[:, 0], res[:, 1], res[:, 2], res[:, 3])
-        
-        
 
 class CsrIntegrator2d(CSRIntegrator):
+
     def __init__(self,
                  image_size,
                  lut=None,
                  empty=0.0,
                  bin_centers0=None,
-                 bin_centers1=None):
+                 bin_centers1=None,
+                 checksum=None,
+                 mask_checksum=None):
         """Constructor of the abstract class for 2D integration
         
         :param size: input image size
@@ -349,12 +403,18 @@ class CsrIntegrator2d(CSRIntegrator):
                      index of the start of line in the CSR matrix
         :param empty: value for empty pixels
         :param bin_center: position of the bin center
+        :param checksum: checksum for the LUT, if not provided, recalculated
+        :param mask_checksum: just a place-holder to track which mask was used
 
         Nota: bins are deduced from bin_centers0, bin_centers1 
     
         """
         self.bin_centers0 = bin_centers0
         self.bin_centers1 = bin_centers1
+        if not checksum:
+            self.checksum = calc_checksum(lut[0])
+        else:
+            self.checksum = checksum
         CSRIntegrator.__init__(self, image_size, lut, empty)
 
     def set_matrix(self, data, indices, indptr):
@@ -371,6 +431,7 @@ class CsrIntegrator2d(CSRIntegrator):
     def integrate(self,
                   signal,
                   variance=None,
+                  poissonian=False,
                   dummy=None,
                   delta_dummy=None,
                   dark=None,
@@ -383,6 +444,7 @@ class CsrIntegrator2d(CSRIntegrator):
         
         :param signal: array of the right size with the signal in it.
         :param variance: Variance associated with the signal
+        :param poissonian: set to True to variance=max(signal,1), False will implement azimuthal variance
         :param dummy: values which have to be discarded (dynamic mask)
         :param delta_dummy: precision for dummy values
         :param dark: noise to be subtracted from signal
@@ -391,14 +453,14 @@ class CsrIntegrator2d(CSRIntegrator):
         :param polarization: :solidangle normalization array
         :param absorption: :absorption normalization array
         :param normalization_factor: scale all normalization with this scalar
-        :return: Integrate2dResult or Integrate2dWithErrorResult object depending is variance is provided 
+        :return: Integrate2dtpl namedtuple: "radial azimuthal intensity error signal variance normalization count"
         
         """
-        if variance is None:
+        if variance is None and poissonian is None:
             do_variance = False
         else:
             do_variance = True
-        trans = CSRIntegrator.integrate(self, signal, variance, dummy, delta_dummy,
+        trans = CSRIntegrator.integrate(self, signal, variance, poissonian, dummy, delta_dummy,
                                         dark, flat, solidangle, polarization,
                                         absorption, normalization_factor)
         trans.shape = self.bins + (-1,)
@@ -420,3 +482,8 @@ class CsrIntegrator2d(CSRIntegrator):
                               intensity, error,
                               signal, variance, normalization, count)
 
+    integrate_ng = integrate
+
+    @property
+    def check_mask(self):
+        return self.mask_checksum is not None
