@@ -1,7 +1,7 @@
 # coding: utf-8
 # /*##########################################################################
 #
-# Copyright (C) 2016-2018 European Synchrotron Radiation Facility
+# Copyright (C) 2016-2021 European Synchrotron Radiation Facility
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -23,11 +23,9 @@
 #
 # ###########################################################################*/
 
-from __future__ import absolute_import
-
 __authors__ = ["V. Valls"]
 __license__ = "MIT"
-__date__ = "28/01/2020"
+__date__ = "26/08/2021"
 
 import logging
 import numpy
@@ -164,8 +162,12 @@ class CalibrationState(qt.QObject):
         return self.__tth
 
     def getRings(self):
+        return [i[1] for i in self.__rings]
+
+    def getIndexedRings(self):
         return self.__rings
 
+
     def getBeamCenter(self):
         return self.__beamCenter
 
@@ -211,7 +213,7 @@ class CalibrationState(qt.QObject):
         mask = calibration.getMask()
         self.__geoRef = calibration.getPyfaiGeometry()
         self.__geometry = None
-        self.__rings = calibration.getRings()
+        self.__rings = calibration.getIndexedRings()
         self.__previousRms = self.__rms
         self.__rms = calibration.getRms()
         tth = calibration.getTwoThetaArray()
@@ -241,7 +243,7 @@ class _RingPlot(silx.gui.plot.PlotWidget):
         self.__axisOfCurrentView = None
         self.__state = None
         self.__tth = None
-        self.__rings = []
+        self.__rings = {}
         self.__ringItems = {}
         self.__angleUnderMouse = None
         self.__displayedAngles = []
@@ -330,7 +332,7 @@ class _RingPlot(silx.gui.plot.PlotWidget):
         result = None
         iresult = None
         minDistance = float("inf")
-        for ringId, ringAngle in enumerate(self.__rings):
+        for ringId, ringAngle in self.__rings.items():
             distance = abs(angle - ringAngle)
             if distance < minDistance:
                 minDistance = distance
@@ -408,7 +410,7 @@ class _RingPlot(silx.gui.plot.PlotWidget):
 
     def __getAvailableAngles(self, minTth, maxTth):
         result = []
-        for ringId, angle in enumerate(self.__rings):
+        for ringId, angle in self.__rings.items():
             if minTth is None or maxTth is None:
                 result.append(ringId, angle)
             if minTth <= angle <= maxTth:
@@ -483,7 +485,7 @@ class _RingPlot(silx.gui.plot.PlotWidget):
                 self.removeCurve(item.getLegend())
         self.__ringItems = {}
         self.__tth = None
-        self.__rings = []
+        self.__rings = {}
 
     def __cleanupMarkers(self):
         try:
@@ -538,7 +540,7 @@ class _RingPlot(silx.gui.plot.PlotWidget):
         if state.isEmpty():
             return
 
-        rings = state.getRings()
+        rings = state.getIndexedRings()
         tth = state.getTwoThetaArray()
         self.__tth = tth
         self.__rings = rings
@@ -861,7 +863,7 @@ class GeometryTask(AbstractCalibrationTask):
         self._fitButton.setWaiting(False)
 
     def __initGeometry(self):
-        self.__initGeometryFromPeaks()
+        self.__initGeometryFromPeaks(useFittedGeometry=True)
 
         # Save this geometry into the history
         calibration = self.__getCalibration()
