@@ -28,7 +28,7 @@
 
 __authors__ = ["Jérôme Kieffer", "Giannis Ashiotis"]
 __license__ = "MIT"
-__date__ = "01/02/2022"
+__date__ = "24/03/2022"
 __copyright__ = "2014-2021, ESRF, Grenoble"
 __contact__ = "jerome.kieffer@esrf.fr"
 
@@ -82,7 +82,9 @@ class OCL_CSR_Integrator(OpenclProcessing):
                numpy.int16: "s16_to_float",
                numpy.uint16: "u16_to_float",
                numpy.uint32: "u32_to_float",
-               numpy.int32: "s32_to_float"
+               numpy.uintc: "u32_to_float",
+               numpy.int32: "s32_to_float",
+               numpy.intc: "s32_to_float"
                }
 
     def __init__(self, lut, image_size, checksum=None,
@@ -320,6 +322,7 @@ class OCL_CSR_Integrator(OpenclProcessing):
                                                             ("data", self.cl_mem["data"]),
                                                             ("indices", self.cl_mem["indices"]),
                                                             ("indptr", self.cl_mem["indptr"]),
+                                                            ("nbins",  numpy.int32(self.bins)),
                                                             ("do_dummy", numpy.int8(0)),
                                                             ("dummy", numpy.float32(0)),
                                                             ("coef_power", numpy.int32(1)),
@@ -353,6 +356,7 @@ class OCL_CSR_Integrator(OpenclProcessing):
                                                             ("data", self.cl_mem["data"]),
                                                             ("indices", self.cl_mem["indices"]),
                                                             ("indptr", self.cl_mem["indptr"]),
+                                                            ("nbins",  numpy.int32(self.bins)),
                                                             ("empty", numpy.float32(self.empty)),
                                                             ("azimuthal", numpy.int8(1)),
                                                             ("merged8", self.cl_mem["merged8"]),
