@@ -30,17 +30,13 @@
 FIXME : make some tests that the functions do what is expected
 """
 
-from __future__ import absolute_import, division, print_function
-
 __author__ = "Jérôme Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "09/05/2019"
-
+__date__ = "20/08/2021"
 
 import unittest
-import sys
 import random
 import time
 import numpy
@@ -59,11 +55,6 @@ from ..third_party import transformations
 from .utilstest import UtilsTest
 import fabio
 
-if sys.platform == "win32":
-    timer = time.clock
-else:
-    timer = time.time
-
 
 class TestSolidAngle(unittest.TestCase):
     """
@@ -100,21 +91,22 @@ class TestSolidAngle(unittest.TestCase):
         data = fabio.open(pilatusFile).data
         data[data < 0] = 0  # discard negative pixels
 
-        tth, I_nogood = ai.integrate1d(data, 1770, unit="2th_deg", radial_range=[0, 56], method="splitBBox", correctSolidAngle=False)
+        method = ("bbox", "histogram", "cython")
+        tth, I_nogood = ai.integrate1d_ng(data, 1770, unit="2th_deg", radial_range=[0, 56], method=method, correctSolidAngle=False)
         delta_tth = abs(tth - tth_fit2d).max()
         delta_I = abs(I_nogood - I_fit2d).max()
         mean_I = abs(I_nogood - I_fit2d).mean()
-        self.assertTrue(delta_tth < 1e-5, 'Error on 2th position: %s <1e-5' % delta_tth)
-        self.assertTrue(delta_I > 100, 'Error on (wrong) I are large: %s >100' % delta_I)
-        self.assertTrue(mean_I > 2, 'Error on (wrong) I are large: %s >2' % mean_I)
+        self.assertLess(delta_tth, 1e-5, 'Error on 2th position: %s <1e-5' % delta_tth)
+        self.assertGreater(delta_I, 100, 'Error on (wrong) I are large: %s >100' % delta_I)
+        self.assertGreater(mean_I, 2, 'Error on (wrong) I are large: %s >2' % mean_I)
 
-        tth, I_good = ai.integrate1d(data, 1770, unit="2th_deg", radial_range=[0, 56], method="splitBBox", correctSolidAngle=3)
+        tth, I_good = ai.integrate1d_ng(data, 1770, unit="2th_deg", radial_range=[0, 56], method=method, correctSolidAngle=3)
         delta_tth = abs(tth - tth_fit2d).max()
         delta_I = abs(I_good - I_fit2d).max()
         mean_I = abs(I_good - I_fit2d).mean()
-        self.assertTrue(delta_tth < 1e-5, 'Error on 2th position: %s <1e-5' % delta_tth)
-        self.assertTrue(delta_I < 5, 'Error on (good) I are small: %s <5' % delta_I)
-        self.assertTrue(mean_I < 0.05, 'Error on (good) I are small: %s <0.05' % mean_I)
+        self.assertLess(delta_tth, 1e-5, 'Error on 2th position: %s <1e-5' % delta_tth)
+        self.assertLess(delta_I, 5, 'Error on (good) I are small: %s <5' % delta_I)
+        self.assertLess(mean_I, 0.05, 'Error on (good) I are small: %s <0.05' % mean_I)
         ai.reset()
 
     def test_nonflat_center(self):
@@ -126,12 +118,14 @@ class TestSolidAngle(unittest.TestCase):
         aarhus = detector_factory("Aarhus")
         aarhus.binning = (10, 10)
         ai = AzimuthalIntegrator(aarhus.radius, detector=aarhus)
-        cosa = numpy.fromfunction(ai.cos_incidence,
-                                  aarhus.shape, dtype=numpy.float32)
-        maxi = cosa.max()
-        mini = cosa.min()
-        self.assertTrue(maxi <= 1.0, 'Cos incidence is %s <=1.0' % maxi)
-        self.assertTrue(mini > 0.99, 'Cos solid angle is %s >0.99' % mini)
+        for path in ("cython", "numexpr", "numpy"):
+            cosa = numpy.fromfunction(ai.cos_incidence,
+                                      aarhus.shape, dtype=numpy.float32,
+                                      path=path)
+            maxi = cosa.max()
+            mini = cosa.min()
+            self.assertLessEqual(maxi, 1.0, f'path:{path} Cos incidence is {maxi} <=1.0')
+            self.assertTrue(mini > 0.99, f'path:{path} Cos incidence is {mini} >0.99')
 
     def test_nonflat_outside(self):
         """
@@ -142,13 +136,15 @@ class TestSolidAngle(unittest.TestCase):
         aarhus = detector_factory("Aarhus")
         aarhus.binning = (10, 10)
         ai = AzimuthalIntegrator(aarhus.radius * 1.5, detector=aarhus)
-        cosa = numpy.fromfunction(ai.cos_incidence,
-                                  aarhus.shape, dtype=numpy.float32)
-        maxi = cosa.max()
-        mini = cosa.min()
-        self.assertTrue(maxi <= 1.0, 'Cos incidence is %s <=1.0' % maxi)
-        self.assertTrue(maxi > 0.99, 'Cos incidence max is %s >0.99' % maxi)
-        self.assertTrue(mini < 0.92, 'Cos solid angle min is %s <0.92' % mini)
+        for path in ("cython", "numexpr", "numpy"):
+            cosa = numpy.fromfunction(ai.cos_incidence,
+                                      aarhus.shape, dtype=numpy.float32,
+                                      path=path)
+            maxi = cosa.max()
+            mini = cosa.min()
+            self.assertLessEqual(maxi, 1.0, f'path: {path} Cos incidence is {maxi} <=1.0')
+            self.assertGreater(maxi, 0.99, f'path: {path} Cos incidence max is {maxi} >0.99')
+            self.assertLess(mini, 0.92, f'path: {path} Cos incidence min is {mini} <0.92')
 
     def test_nonflat_inside(self):
         """
@@ -159,14 +155,31 @@ class TestSolidAngle(unittest.TestCase):
         aarhus = detector_factory("Aarhus")
         aarhus.binning = (10, 10)
         ai = AzimuthalIntegrator(aarhus.radius * 0.5, detector=aarhus)
-        cosa = numpy.fromfunction(ai.cos_incidence,
-                                  aarhus.shape, dtype=numpy.float32)
-        maxi = cosa.max()
-        mini = cosa.min()
-        self.assertTrue(maxi <= 1.0, 'Cos incidence is %s <=1.0' % maxi)
-        self.assertTrue(maxi > 0.99, 'Cos incidence max is %s >0.99' % maxi)
-        self.assertTrue(mini < 0.87, 'Cos solid angle min is %s <0.86' % mini)
+        for path in ("cython", "numexpr", "numpy"):
+            cosa = numpy.fromfunction(ai.cos_incidence,
+                                      aarhus.shape, dtype=numpy.float32,
+                                      path=path)
+            maxi = cosa.max()
+            mini = cosa.min()
+            self.assertLessEqual(maxi, 1.0, f'path: {path} Cos incidence is {maxi} <=1.0')
+            self.assertGreater(maxi, 0.99, f'path: {path} Cos incidence max is {maxi} >0.99')
+            self.assertLess(mini, 0.87, f'path: {path} Cos incidence min is {mini} <0.86')
 
+    def test_flat(self):
+        """test sine and cosine for the incidence angle
+        """
+        pilatus = detector_factory("Pilatus100k")
+        ai = AzimuthalIntegrator(0.1, detector=pilatus)
+        for path in ("cython", "numexpr", "numpy"):
+            cosa = numpy.fromfunction(ai.cos_incidence,
+                                      pilatus.shape, dtype=numpy.float64,
+                                      path=path)
+            sina = numpy.fromfunction(ai.sin_incidence,
+                                      pilatus.shape, dtype=numpy.float64,
+                                      path=path)
+            one = cosa*cosa + sina*sina
+            self.assertLessEqual(one.max()-1.0, 1e-10, f"path: {path} cos2+sin2<=1")
+            self.assertGreater(one.min()-1.0, -1e-10, f"path: {path} cos2+sin2>0.99")
 
 class TestBug88SolidAngle(unittest.TestCase):
     """
@@ -176,10 +189,11 @@ class TestBug88SolidAngle(unittest.TestCase):
     """
 
     def testSolidAngle(self):
+        method = ("no", "histogram", "numpy")
         img = numpy.ones((1000, 1000), dtype=numpy.float32)
         ai = AzimuthalIntegrator(dist=0.01, detector="Titan", wavelength=1e-10)
-        t = ai.integrate1d(img, 1000, method="numpy")[1].max()
-        f = ai.integrate1d(img, 1000, method="numpy", correctSolidAngle=False)[1].max()
+        t = ai.integrate1d_ng(img, 1000, method=method)[1].max()
+        f = ai.integrate1d_ng(img, 1000, method=method, correctSolidAngle=False)[1].max()
         self.assertAlmostEqual(f, 1, 5, "uncorrected flat data are unchanged")
         self.assertNotAlmostEqual(f, t, 1, "corrected and uncorrected flat data are different")
 
@@ -187,6 +201,7 @@ class TestBug88SolidAngle(unittest.TestCase):
 class TestRecprocalSpacingSquarred(unittest.TestCase):
     """
     """
+
     def setUp(self):
         from pyFAI.detectors import Detector
         self.shape = (50, 49)
@@ -208,15 +223,15 @@ class TestRecprocalSpacingSquarred(unittest.TestCase):
         self.assertTrue(numpy.allclose(rd2, (q / (2 * numpy.pi)) ** 2), "center rd2 = (q/2pi)**2")
 
     def test_corner(self):
-        rd2 = self.geo.corner_array(self.shape, unit=units.RecD2_NM, scale=False)[:, :, :, 0]
-        q = self.geo.corner_array(self.shape, unit=units.Q, use_cython=False, scale=False)[:, :, :, 0]
+        rd2 = self.geo.corner_array(self.shape, unit=units.RecD2_NM, scale=False)[:,:,:, 0]
+        q = self.geo.corner_array(self.shape, unit=units.Q, use_cython=False, scale=False)[:,:,:, 0]
         delta = rd2 - (q / (2 * numpy.pi)) ** 2
         self.assertTrue(numpy.allclose(rd2, (q / (2 * numpy.pi)) ** 2), "corners rd2 = (q/2pi)**2, delat=%s" % delta)
 
     def test_delta(self):
         drd2a = self.geo.deltaRd2(self.shape)
         rd2 = self.geo.rd2Array(self.shape)
-        rc = self.geo.corner_array(self.shape, unit=units.RecD2_NM, scale=False)[:, :, :, 0]
+        rc = self.geo.corner_array(self.shape, unit=units.RecD2_NM, scale=False)[:,:,:, 0]
         drd2 = self.geo.deltaRd2(self.shape)
         self.assertTrue(numpy.allclose(drd2, drd2a, atol=1e-5), "delta rd2 = (q/2pi)**2, one formula with another")
         delta2 = abs(rc - numpy.atleast_3d(rd2)).max(axis=-1)
@@ -305,7 +320,7 @@ class TestFastPath(utilstest.ParametricTestCase):
                 geometries.append(dico)
                 q = transformations.quaternion_from_euler(-dico["rot1"], -dico["rot2"], dico["rot3"], axes="sxyz")
                 quaternions.append(q)
-                matrices.append(transformations.quaternion_matrix(q)[:3, :3])
+                matrices.append(transformations.quaternion_matrix(q)[:3,:3])
         cls.geometries = geometries
         cls.quaternions = quaternions
         cls.matrices = matrices
@@ -329,13 +344,13 @@ class TestFastPath(utilstest.ParametricTestCase):
         for data, space in params:
             with self.subTest(data=data, space=space):
                 geo = geometry.Geometry(**data)
-                t00 = timer()
+                t00 = time.perf_counter()
                 py_res = geo.corner_array(unit=space, use_cython=False, scale=False)
-                t01 = timer()
+                t01 = time.perf_counter()
                 geo.reset()
-                t10 = timer()
+                t10 = time.perf_counter()
                 cy_res = geo.corner_array(unit=space, use_cython=True, scale=False)
-                t11 = timer()
+                t11 = time.perf_counter()
                 delta = abs(py_res - cy_res)
                 # We expect precision on radial position
                 delta_r = delta[..., 0].max()
@@ -343,8 +358,8 @@ class TestFastPath(utilstest.ParametricTestCase):
                 cnt_delta_a = (delta[..., 1] > self.EPSILON_A).sum()
                 logger.debug("TIMINGS\t meth: %s %s Python: %.3fs, Cython: %.3fs\t x%.3f\t delta_r:%s",
                              space, data["detector"], t01 - t00, t11 - t10, (t01 - t00) / numpy.float64(t11 - t10), delta)
-                self.assertTrue(delta_r < self.EPSILON_R, "data=%s, space='%s' delta_r: %s" % (data, space, delta_r))
-                self.assertTrue(cnt_delta_a < count_a, "data:%s, space: %s cnt_delta_a: %s" % (data, space, cnt_delta_a))
+                self.assertLess(delta_r, self.EPSILON_R, "data=%s, space='%s' delta_r: %s" % (data, space, delta_r))
+                self.assertLess(cnt_delta_a, count_a, "data:%s, space: %s cnt_delta_a: %s" % (data, space, cnt_delta_a))
 
     def test_XYZ(self):
         """Test the calc_pos_zyx with full detectors"""
@@ -352,11 +367,11 @@ class TestFastPath(utilstest.ParametricTestCase):
         for geometryParams in geometries:
             with self.subTest(geometry=geometry):
                 geo = geometry.Geometry(**geometryParams)
-                t0 = timer()
+                t0 = time.perf_counter()
                 py_res = geo.calc_pos_zyx(corners=True, use_cython=False)
-                t1 = timer()
+                t1 = time.perf_counter()
                 cy_res = geo.calc_pos_zyx(corners=True, use_cython=True)
-                t2 = timer()
+                t2 = time.perf_counter()
                 delta = numpy.array([abs(py - cy).max() for py, cy in zip(py_res, cy_res)])
                 logger.debug("TIMINGS\t meth: calc_pos_zyx %s, corner=True python t=%.3fs\t cython: t=%.3fs \t x%.3f delta %s",
                              geometryParams["detector"], t1 - t0, t2 - t1, (t1 - t0) / numpy.float64(t2 - t1), delta)
@@ -370,13 +385,13 @@ class TestFastPath(utilstest.ParametricTestCase):
         for geometryParams in geometries:
             with self.subTest(geometry=geometryParams):
                 geo = geometry.Geometry(**geometryParams)
-                t0 = timer()
+                t0 = time.perf_counter()
                 py_res = geo.deltaChi(use_cython=False)
-                # t1 = timer()
+                # t1 = time.perf_counter()
                 geo.reset()
-                t1 = timer()
+                t1 = time.perf_counter()
                 cy_res = geo.deltaChi(use_cython=True)
-                t2 = timer()
+                t2 = time.perf_counter()
                 delta = numpy.array([abs(py - cy).max() for py, cy in zip(py_res, cy_res)])
                 logger.debug("TIMINGS\t meth: deltaChi %s python t=%.3fs\t cython: t=%.3fs \t x%.3f delta %s",
                              geometryParams["detector"], t1 - t0, t2 - t1, (t1 - t0) / numpy.float64(t2 - t1), delta)
@@ -393,8 +408,8 @@ class TestFastPath(utilstest.ParametricTestCase):
         self.assertEqual(len(geometries), len(matrices), "length is the same")
         for kwds, quat, mat in zip(geometries, quaternions, matrices):
             geo = geometry.Geometry(**kwds)
-            self.assert_(numpy.allclose(geo.rotation_matrix(), mat), "matrice are the same %s" % kwds)
-            self.assert_(numpy.allclose(geo.quaternion(), quat), "quaternions are the same %s" % kwds)
+            self.assertTrue(numpy.allclose(geo.rotation_matrix(), mat), "matrice are the same %s" % kwds)
+            self.assertTrue(numpy.allclose(geo.quaternion(), quat), "quaternions are the same %s" % kwds)
 
 
 class TestGeometry(utilstest.ParametricTestCase):
@@ -416,12 +431,12 @@ class TestGeometry(utilstest.ParametricTestCase):
         pixels = {"detector": "Pilatus100k",
                   "wavelength": 1e-10}
         geometries = [{'dist': 1, 'rot1': 0, 'rot2': 0, 'rot3': 0},
-                      {'dist': 1, 'rot1': -1, 'rot2': 1, 'rot3': 1},
-                      {'dist': 1, 'rot1': -.2, 'rot2': 1, 'rot3': -.1},
-                      {'dist': 1, 'rot1': -1, 'rot2': -.2, 'rot3': 1},
+                      {'dist': 1, 'rot1':-1, 'rot2': 1, 'rot3': 1},
+                      {'dist': 1, 'rot1':-.2, 'rot2': 1, 'rot3':-.1},
+                      {'dist': 1, 'rot1':-1, 'rot2':-.2, 'rot3': 1},
                       {'dist': 1, 'rot1': 1, 'rot2': 5, 'rot3': .4},
-                      {'dist': 1, 'rot1': -1.2, 'rot2': 1, 'rot3': 1},
-                      {'dist': 100, 'rot1': -2, 'rot2': 2, 'rot3': 1},
+                      {'dist': 1, 'rot1':-1.2, 'rot2': 1, 'rot3': 1},
+                      {'dist': 100, 'rot1':-2, 'rot2': 2, 'rot3': 1},
                       ]
         for g in geometries:
             g.update(pixels)
@@ -435,11 +450,11 @@ class TestGeometry(utilstest.ParametricTestCase):
         for func, varargs, kwds in params:
             with self.subTest(function=func, varargs=varargs, kwds=kwds):
                 geo = geometry.Geometry(**kwds)
-                t0 = timer()
+                t0 = time.perf_counter()
                 oldret = getattr(geo, func)(self.D1, self.D2, path=varargs[0])
-                t1 = timer()
+                t1 = time.perf_counter()
                 newret = getattr(geo, func)(self.D1, self.D2, path=varargs[1])
-                t2 = timer()
+                t2 = time.perf_counter()
                 delta = abs(oldret - newret).max()
                 logger.debug("TIMINGS\t %s meth: %s %.3fs\t meth: %s %.3fs, x%.3f delta %s",
                              func, varargs[0], t1 - t0, varargs[1], t2 - t1, (t1 - t0) / numpy.float64(t2 - t1), delta)
@@ -454,11 +469,11 @@ class TestGeometry(utilstest.ParametricTestCase):
         for corners, kwds in params:
             with self.subTest(corners=corners, kwds=kwds):
                 geo = geometry.Geometry(**kwds)
-                t0 = timer()
+                t0 = time.perf_counter()
                 py_res = geo.calc_pos_zyx(None, self.D1, self.D2, corners=corners, use_cython=False)
-                t1 = timer()
+                t1 = time.perf_counter()
                 cy_res = geo.calc_pos_zyx(None, self.D1, self.D2, corners=corners, use_cython=True)
-                t2 = timer()
+                t2 = time.perf_counter()
                 delta = numpy.array([abs(py - cy).max() for py, cy in zip(py_res, cy_res)])
                 logger.debug("TIMINGS\t meth: calc_pos_zyx, corner=%s python t=%.3fs\t cython: t=%.3fs\t x%.3f delta %s",
                              corners, t1 - t0, t2 - t1, (t1 - t0) / numpy.float64(t2 - t1), delta)
@@ -485,10 +500,10 @@ class TestCalcFrom(unittest.TestCase):
     def test_calcfrom12d(self):
         det = detector_factory("pilatus300k")
         ai = AzimuthalIntegrator(0.1, 0.05, 0.04, detector=det)
-        prof_1d = ai.integrate1d(numpy.random.random(det.shape), 200, unit="2th_deg")
+        prof_1d = ai.integrate1d_ng(numpy.random.random(det.shape), 200, unit="2th_deg")
         sig = numpy.sinc(prof_1d.radial * 10) ** 2
         img1 = ai.calcfrom1d(prof_1d.radial, sig, dim1_unit="2th_deg", mask=det.mask, dummy=-1)
-        new_prof_1d = ai.integrate1d(img1, 200, unit="2th_deg")
+        new_prof_1d = ai.integrate1d_ng(img1, 200, unit="2th_deg")
         delta = abs((new_prof_1d.intensity - sig)).max()
         self.assertLess(delta, 2e-3, "calcfrom1d works delta=%s" % delta)
         prof_2d = ai.integrate2d(img1, 400, 360, unit="2th_deg")
