@@ -26,8 +26,6 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
-from __future__ import absolute_import, division, print_function
-
 """Test suites for pixel splitting scheme validation
 
 see sandbox/debug_split_pixel.py for visual validation
@@ -37,9 +35,10 @@ __author__ = "Jérôme Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "30/11/2018"
+__date__ = "17/01/2022"
 
 import unittest
+import platform
 import numpy
 import logging
 from .utilstest import UtilsTest
@@ -48,6 +47,57 @@ from ..azimuthalIntegrator import AzimuthalIntegrator
 from ..detectors import Detector
 from ..utils import mathutil
 from ..ext import splitBBox, splitPixel
+from ..method_registry import IntegrationMethod
+
+
+class TestRecenter(unittest.TestCase):
+
+    """See sandbox/PixelSplitting.ipynb"""
+
+    def test_disc0(self):
+        disc_at_pi = 0
+        detector = Detector(1e-3, 1e-3, max_shape=(5, 5))
+        ai = AzimuthalIntegrator(1, 2.2e-3, 2.8e-3, rot3=0.5, detector=detector)
+        ai.setChiDiscAtZero()
+        pos = ai.array_from_unit(typ="corner", unit="r_mm", scale=True).astype(splitPixel.position_d)
+        area = []
+        for i0 in range(pos.shape[0]):
+            for i1 in range(pos.shape[1]):
+                area.append(splitPixel.recenter(pos[i0, i1], chiDiscAtPi=disc_at_pi))
+        self.assertLessEqual(max(area), 0, "All area are negative")
+        self.assertEqual((pos[..., 1] > 2 * numpy.pi).sum(), 3, "3 corner is >2pi")
+        self.assertEqual((pos[..., 1] < 0).sum(), 1, "1 corner are <0")
+
+    def test_discpi(self):
+        detector = Detector(1e-3, 1e-3, max_shape=(5, 5))
+        ai = AzimuthalIntegrator(1, 2.2e-3, 2.8e-3, rot3=-0.4, detector=detector)
+
+        disc_at_pi = 2
+        ai.setChiDiscAtPi()
+        pos = ai.array_from_unit(typ="corner", unit="r_mm", scale=True).astype(splitPixel.position_d)
+        area = []
+        for i0 in range(pos.shape[0]):
+            for i1 in range(pos.shape[1]):
+                area.append(splitPixel.recenter(pos[i0, i1], chiDiscAtPi=disc_at_pi))
+        self.assertLessEqual(max(area), 0, "All area are negative")
+        print((pos[..., 1] > numpy.pi).sum(), (pos[..., 1] < -numpy.pi).sum())
+        self.assertEqual((pos[..., 1] > numpy.pi).sum(), 1, "1 corner is >pi")
+        self.assertEqual((pos[..., 1] < -numpy.pi).sum(), 5, "5 corner are <-pi")
+
+    def test_area(self):
+        "Test the formula to calculate the area of any quad"
+        pos = numpy.random.random(8) * 10  # this is a random quad !
+
+        ref = splitPixel._sp_area4(*pos)
+        trp = splitPixel._sp_area4(*pos.reshape((-1, 2))[:, -1::-1].ravel())
+        self.assertAlmostEqual(ref, trp, msg="Check transposed order")
+
+        b = numpy.concatenate((pos, pos))
+        buf = numpy.zeros(int(max(pos) + 3), numpy.float32)
+        for i in range(4):
+            splitPixel._sp_integrate1d(buf, *b[2 * i:2 * i + 4])
+        print(buf, buf.sum(), ref, trp)
+        self.assertAlmostEqual(abs(buf.sum()), ref, 4, "Check integration")
 
 
 class TestSplitPixel(unittest.TestCase):
@@ -62,15 +112,40 @@ class TestSplitPixel(unittest.TestCase):
         det.shape = (512, 512)
         ai = AzimuthalIntegrator(1, detector=det)
         cls.results = {}
+        cls.results_ng = {}
         for i, meth in enumerate(["numpy", "cython", "splitbbox", "splitpixel", "csr_no", "csr_bbox", "csr_full"]):
-            cls.results[meth] = ai.integrate1d(img, 10000, method=meth, unit="2th_deg")
+            cls.results[meth] = ai.integrate1d_legacy(img, 10000, method=meth, unit="2th_deg")
             ai.reset()
+        for k, v in IntegrationMethod._registry.items():
+            if v.dimension == 1 and  v.target is None:  # exclude OpenCL engines
+                cls.results_ng[k] = ai.integrate1d_ng(img, 10000, method=v, unit="r_mm")
 
     @classmethod
     def tearDownClass(cls):
         super(TestSplitPixel, cls).tearDownClass()
         cls.results = None
 
+    def test_new_gen_algoritms(self):
+        "This checks that the pixel splitting scheme gives consistent results"
+        self.assertGreater(len(self.results_ng), 0, msg="we have some results")
+        thres = 7
+        for k1, res1 in self.results_ng.items():
+            if k1.split == "pseudo":
+                # Those are half implemented algorithms ... avoid testing them!
+                continue
+            for k2, res2 in self.results_ng.items():
+                if k1 == k2:
+                    continue
+                if k2.split == "pseudo":
+                    continue
+                R = mathutil.rwp(res1, res2)
+                print (f"({k1.split},{k1.algo})/({k2.split},{k2.algo})\t {R}")
+
+                if k1.split == k2.split:
+                    self.assertTrue(R < thres, f"{k1}/{k2}")
+                else:
+                    self.assertTrue(R > thres, f"{k1}/{k2}")
+
     def test_no_split(self):
         """
         Validate that all non splitting algo give the same result...
@@ -130,13 +205,20 @@ class TestSplitPixel(unittest.TestCase):
 class TestSplitBBoxNg(unittest.TestCase):
     """Test the equivalence of the historical SplitBBox with the one propagating 
     the variance"""
+
     @classmethod
     def setUpClass(cls):
         super(TestSplitBBoxNg, cls).setUpClass()
         det = Detector.factory("Pilatus 100k")
         shape = det.shape
         # The randomness of the image is not correlated to bug #1021
-        img = numpy.random.randint(0, 65000, numpy.prod(shape))
+        cls.maxi = 65000
+        img = numpy.random.randint(0, cls.maxi, numpy.prod(shape))
+
+        if platform.machine() in ("i386", "i686", "x86_64") and (tuple.__itemsize__ == 4):
+            cls.epsilon = 1e-13
+        else:
+            cls.epsilon = numpy.finfo(numpy.float64).eps
 
         ai = AzimuthalIntegrator(1, detector=det)
         ai.wavelength = 1e-10
@@ -181,59 +263,60 @@ class TestSplitBBoxNg(unittest.TestCase):
     def test_split_bbox_2d(self):
         # radial position:
         tth_legacy = self.results["histoBBox2d_legacy"][1]
-        tth_ng = self.results["histoBBox2d_ng"][2]
+        tth_ng = self.results["histoBBox2d_ng"].radial
         self.assertEqual(abs(tth_legacy - tth_ng).max(), 0, "radial position is the same")
 
         # azimuthal position:
         chi_legacy = self.results["histoBBox2d_legacy"][2]
-        chi_ng = self.results["histoBBox2d_ng"][3]
+        chi_ng = self.results["histoBBox2d_ng"].azimuthal
         self.assertEqual(abs(chi_legacy - chi_ng).max(), 0, "azimuthal position is the same")
 
         # pixel count:
         count_legacy = self.results["histoBBox2d_legacy"][4]
-        count_ng = self.results["histoBBox2d_ng"][4]["count"]
+        count_ng = self.results["histoBBox2d_ng"].count
 
         if abs(count_ng).max() == 0:
             print(splitBBox)
             print(count_legacy)
             print(count_ng)
-            print("prop", self.results["histoBBox2d_ng"][4])
-            print("pos1", self.results["histoBBox2d_ng"][3])
-            print("pos0", self.results["histoBBox2d_ng"][2])
-            print("err", self.results["histoBBox2d_ng"][1])
-            print("int", self.results["histoBBox2d_ng"][0])
-        self.assertEqual(abs(count_legacy - count_ng).max(), 0, "count is the same")
+#             print("prop", self.results["histoBBox2d_ng"][4])
+#             print("pos1", self.results["histoBBox2d_ng"][3])
+#             print("pos0", self.results["histoBBox2d_ng"][2])
+#             print("err", self.results["histoBBox2d_ng"][1])
+#             print("int", self.results["histoBBox2d_ng"][0])
+
+        self.assertLess(abs(count_legacy - count_ng).max(), self.epsilon, "count is the same")
         # same for normalisation ... in this case
-        count_ng = self.results["histoBBox2d_ng"][4]["norm"]
-        self.assertEqual(abs(count_legacy - count_ng).max(), 0, "norm is old-count")
+        count_ng = self.results["histoBBox2d_ng"].normalization
+        self.assertLess(abs(count_legacy - count_ng).max(), self.epsilon, "norm is old-count")
 
         # Weighted signal:
         weighted_legacy = self.results["histoBBox2d_legacy"][3]
-        signal = self.results["histoBBox2d_ng"][4]["signal"]
-        self.assertEqual(abs(signal - weighted_legacy).max(), 0, "Weighted is the same")
+        signal = self.results["histoBBox2d_ng"].signal
+        self.assertLess(abs(signal - weighted_legacy).max(), self.maxi * self.epsilon, "Weighted is the same")
 
         # resulting intensity validation
         int_legacy = self.results["histoBBox2d_legacy"][0]
-        int_ng = self.results["histoBBox2d_ng"][0]
-        self.assertEqual(abs(int_legacy - int_ng).max(), 0, "intensity is the same")
+        int_ng = self.results["histoBBox2d_ng"].intensity
+        self.assertLess(abs(int_legacy - int_ng).max(), self.epsilon, "intensity is the same")
 
     def test_split_pixel_2d(self):
         # radial position:
         tth_legacy = self.results["fullSplit2D_legacy"][1]
-        tth_ng = self.results["fullSplit2D_ng"][2]
+        tth_ng = self.results["fullSplit2D_ng"].radial
         self.assertEqual(abs(tth_legacy - tth_ng).max(), 0, "radial position is the same")
 
         # azimuthal position:
         chi_legacy = self.results["fullSplit2D_legacy"][2]
-        chi_ng = self.results["fullSplit2D_ng"][3]
+        chi_ng = self.results["fullSplit2D_ng"].azimuthal
         self.assertEqual(abs(chi_legacy - chi_ng).max(), 0, "azimuthal position is the same")
 
         # pixel count:
         count_legacy = self.results["fullSplit2D_legacy"][4]
-        count_ng = self.results["fullSplit2D_ng"][4]["count"]
+        count_ng = self.results["fullSplit2D_ng"].count
         self.assertLess(abs(count_legacy - count_ng).mean(), 1, "count is almost the same")
         # same for normalisation ... in this case
-        count_ng = self.results["fullSplit2D_ng"][4]["norm"]
+        count_ng = self.results["fullSplit2D_ng"].normalization
         self.assertLess(abs(count_legacy - count_ng).mean(), 1, "norm is almost old-count")
 
 #         # Weighted signal:
@@ -257,6 +340,7 @@ def suite():
     testsuite = unittest.TestSuite()
     testsuite.addTest(loader(TestSplitPixel))
     testsuite.addTest(loader(TestSplitBBoxNg))
+    testsuite.addTest(loader(TestRecenter))
     return testsuite
 
 
