@@ -23,16 +23,14 @@
 # THE SOFTWARE.
 #
 "bunch of utility function/static classes to handle testing environment"
-from __future__ import print_function, division, absolute_import, with_statement
 
 __author__ = "Jérôme Kieffer"
 __contact__ = "jerome.kieffer@esrf.eu"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "09/10/2018"
+__date__ = "10/01/2022"
 
 PACKAGE = "pyFAI"
-DATA_KEY = "PYFAI_DATA"
 
 import os
 import sys
@@ -45,10 +43,8 @@ import tempfile
 import getpass
 import functools
 
-
-from argparse import ArgumentParser
-from ..third_party import six
 from silx.resources import ExternalResources
+from ..directories import testimages
 
 logger = logging.getLogger(__name__)
 
@@ -67,6 +63,7 @@ class TestOptions(object):
     """
     Class providing useful stuff for preparing tests.
     """
+
     def __init__(self):
         self.WITH_QT_TEST = True
         """Qt tests are included"""
@@ -92,7 +89,7 @@ class TestOptions(object):
         self.url_base = "http://ftp.edna-site.org/pyFAI/testimages"
         self.resources = ExternalResources(PACKAGE,
                                            timeout=self.timeout,
-                                           env_key=DATA_KEY,
+                                           env_key=testimages,
                                            url_base=self.url_base)
         self.sem = threading.Semaphore()
         self.recompiled = False
@@ -133,11 +130,11 @@ class TestOptions(object):
         """
         return
 
-    def configure(self, parsed_options):
+    def configure(self, parsed_options=None):
         """Configure the TestOptions class from the command line arguments and the
         environment variables
         """
-        if not parsed_options.gui:
+        if parsed_options is not None and not parsed_options.gui:
             self.WITH_QT_TEST = False
             self.WITH_QT_TEST_REASON = "Skipped by command line"
         elif os.environ.get('WITH_QT_TEST', 'True') == 'False':
@@ -147,20 +144,27 @@ class TestOptions(object):
             self.WITH_QT_TEST = False
             self.WITH_QT_TEST_REASON = "DISPLAY env variable not set"
 
-        if not parsed_options.opencl or os.environ.get('PYFAI_OPENCL', 'True') == 'False':
+        if parsed_options is not None and not parsed_options.opencl:
+            self.WITH_OPENCL_TEST = False
+            # That's an easy way to skip OpenCL tests
+            # It disable the use of OpenCL on the full silx project
+            os.environ['PYFAI_OPENCL'] = "False"
+        elif os.environ.get('PYFAI_OPENCL', 'True') == 'False':
             self.WITH_OPENCL_TEST = False
             # That's an easy way to skip OpenCL tests
             # It disable the use of OpenCL on the full silx project
             os.environ['PYFAI_OPENCL'] = "False"
 
-        if not parsed_options.opengl:
+        if parsed_options is not None and not parsed_options.opengl:
             self.WITH_GL_TEST = False
             self.WITH_GL_TEST_REASON = "Skipped by command line"
         elif os.environ.get('WITH_GL_TEST', 'True') == 'False':
             self.WITH_GL_TEST = False
             self.WITH_GL_TEST_REASON = "Skipped by WITH_GL_TEST env var"
 
-        if parsed_options.low_mem or os.environ.get('PYFAI_LOW_MEM', 'True') == 'False':
+        if parsed_options is not None and parsed_options.low_mem:
+            self.TEST_LOW_MEM = True
+        elif os.environ.get('PYFAI_LOW_MEM', 'True') == 'False':
             self.TEST_LOW_MEM = True
 
     def add_parser_argument(self, parser):
@@ -242,6 +246,23 @@ class TestOptions(object):
             self._initialize_tmpdir()
         return self._tempdir
 
+    def tempfile(self, suffix=None, prefix=None, dir=None, text=False):
+        """create a temporary file, openend 
+        
+        See tempfile.mkstemp for the description of the options
+        :param suffix: end of the filename
+        :param prefix: start of the filename
+        :dir: subdir where the file is created
+        :text: create the text in text (or binary) mode 
+        return file_descriptor, filename
+        """
+        dest = self.tempdir
+        if dir is not None:
+            dest = os.path.join(dest, dir)
+            if not os.path.isdir(dest):
+                os.makedirs(dest)
+        return tempfile.mkstemp(suffix, prefix, dir=dest, text=text)
+
     def clean_up(self):
         """Removes the temporary directory (and all its content !)"""
         with self.sem:
@@ -291,7 +312,7 @@ def diff_img(ref, obt, comment=""):
         y = imax // ref.shape[-1]
         ax3.plot([x], [y], "o", scalex=False, scaley=False)
         fig.show()
-        six.moves.input()
+        input()
 
 
 def diff_crv(ref, obt, comment=""):
@@ -309,44 +330,11 @@ def diff_crv(ref, obt, comment=""):
         _im_obt = ax1.plot(obt, label="%s obt" % comment)
         _im_delta = ax2.plot(delta, label="delta")
         fig.show()
-        six.moves.input()
+        input()
 
 
-if sys.hexversion >= 0x030400F0:  # Python >= 3.4
-    class ParametricTestCase(unittest.TestCase):
-        pass
+ParametricTestCase = unittest.TestCase
 
-else:
-    class ParametricTestCase(unittest.TestCase):
-        """TestCase with subTest support for Python < 3.4.
-
-        Add subTest method to support parametric tests.
-        API is the same, but behavior differs:
-        If a subTest fails, the following ones are not run.
-        """
-
-        _subtest_msg = None  # Class attribute to provide a default value
-
-        @contextlib.contextmanager
-        def subTest(self, msg=None, **params):
-            """Use as unittest.TestCase.subTest method in Python >= 3.4."""
-            # Format arguments as: '[msg] (key=value, ...)'
-            param_str = ', '.join(['%s=%s' % (k, v) for k, v in params.items()])
-            self._subtest_msg = '[%s] (%s)' % (msg or '', param_str)
-            yield
-            self._subtest_msg = None
-
-        def shortDescription(self):
-            short_desc = super(ParametricTestCase, self).shortDescription()
-            if self._subtest_msg is not None:
-                # Append subTest message to shortDescription
-                short_desc = ' '.join(
-                    [msg for msg in (short_desc, self._subtest_msg) if msg])
-
-            return short_desc if short_desc else None
-
-
-# Test logging messages #######################################################
 
 class TestLogging(logging.Handler):
     """Context checking the number of logging messages from a specified Logger.
@@ -463,6 +451,7 @@ def test_logging(logger=None, critical=None, error=None,
     :param int notset: Expected number of NOTSET messages.
                        Default: Do not check.
     """
+
     def decorator(func):
         test_context = TestLogging(logger, critical, error,
                                    warning, info, debug, notset)
@@ -472,7 +461,9 @@ def test_logging(logger=None, critical=None, error=None,
             with test_context:
                 result = func(*args, **kwargs)
             return result
+
         return wrapper
+
     return decorator
 
 
@@ -482,13 +473,14 @@ def create_fake_data(dist=1, poni1=0, poni2=0, rot1=0, rot2=0, rot3=0,
     """Simulate a SAXS image with a small detector by default
     :return: image, azimuthalIngtegrator 
     """
-    import pyFAI.calibrant
-    import pyFAI.azimuthalIntegrator
+    from .. import calibrant as pyFAI_calibrant
+    from .. import azimuthalIntegrator
     import numpy
-    cal = pyFAI.calibrant.get_calibrant("AgBh")
+    cal = pyFAI_calibrant.get_calibrant(calibrant)
     cal.wavelength = wavelength
-    ai = pyFAI.azimuthalIntegrator.AzimuthalIntegrator(dist, poni1, poni2,
-                    rot1, rot2, rot3, detector=detector, wavelength=wavelength)
+    ai = azimuthalIntegrator.AzimuthalIntegrator(dist, poni1, poni2,
+                                                 rot1, rot2, rot3,
+                                                 detector=detector, wavelength=wavelength)
     img = cal.fake_calibration_image(ai, Imax=Imax) + offset
     if poissonian:
         return numpy.random.poisson(img), ai
